/*****************************************************
*
*	Control program for the PitSchuLight TV-Backlight
*	(c) Peter Schulten, Mlheim, Germany
*	peter_(at)_pitschu.de
*
*	Die unvernderte Wiedergabe und Verteilung dieses gesamten Sourcecodes
*	in beliebiger Form ist gestattet, sofern obiger Hinweis erhalten bleibt.
*
* 	Ich stelle diesen Sourcecode kostenlos zur Verfgung und biete daher weder
*	Support an noch garantiere ich fr seine Funktionsfhigkeit. Auerdem
*	bernehme ich keine Haftung fr die Folgen seiner Nutzung.

*	Der Sourcecode darf nur zu privaten Zwecken verwendet und modifiziert werden.
*	Darber hinaus gehende Verwendung bedarf meiner Zustimmung.
*
*	History
*	09.06.2013	pitschu		Start of work
*/

#include <stdio.h>
#include "stm32f4xx.h"
#include "ws2812.h"

volatile rgbValue_t 	ws2812ledRGB[WS2812_NR_LEDS];			// color data for all leds
static uint16_t 		ws2812timerValues[WS2812_TIMERDMA_LEN];	// buffer for timer/dma, one byte per bit + reset pulse
volatile uint8_t		ledBusy = 0;							// = 1 while dma is sending data to leds



static void WS2812startDMA(void);									// start the dma transfer (framebuffer to timer)


//-------------------------------------------------------------------------------------------------------------

static inline uint16_t *rgb2pwm(uint16_t *dest, const uint8_t color)
{
	register uint8_t mask = 0x80;

	do
	{
		if (color & mask)
			*dest = WS2812_PWM_ONE;
		else
			*dest = WS2812_PWM_ZERO;

		dest++;
		mask >>= 1;
	} while (mask != 0);

	return dest;
}




// convert RGB values into timer output compare values
void WS2812update(void)
{
	register uint32_t i;
	register rgbValue_t *r;
	uint16_t * bufp = ws2812timerValues;

	for (i = 0; i < WS2812_NR_LEDS; i++)
	{
		r = (rgbValue_t *)&ws2812ledRGB[i];

		bufp = rgb2pwm(bufp, r->G);
		bufp = rgb2pwm(bufp, r->R);
		bufp = rgb2pwm(bufp, r->B);
	}

	WS2812startDMA();		// send it to RGB stripe
}





void WS2812init(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	TIM_TimeBaseInitTypeDef timbaseinit;
	TIM_OCInitTypeDef timocinit;
	NVIC_InitTypeDef nvic_init_struc;

	// clear dma buffer
	int i;

	for (i = 0; i < (WS2812_TIMERDMA_LEN - WS2812_RESET_LEN); i++)
		ws2812timerValues[i] = WS2812_PWM_ZERO;
	for (; i < WS2812_TIMERDMA_LEN; i++)
		ws2812timerValues[i] = 0;

	// GPIO
	RCC_AHB1PeriphClockCmd(WS2812_RCC_GPIO, ENABLE);
	GPIO_InitStructure.GPIO_Pin = WS2812_GPIO_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
	GPIO_Init(WS2812_GPIO, &GPIO_InitStructure);
	GPIO_PinAFConfig(WS2812_GPIO, GPIO_PinSource0, WS2812_GPIO_AF_TIM);

	// TIMER
	RCC_APB1PeriphClockCmd(WS2812_RCC_TIM, ENABLE);

	TIM_TimeBaseStructInit(&timbaseinit);
	timbaseinit.TIM_ClockDivision = 		TIM_CKD_DIV1;
	timbaseinit.TIM_CounterMode = 			TIM_CounterMode_Up;
	timbaseinit.TIM_Period = 				WS2812_TIM_FREQ / WS2812_OUT_FREQ;
	timbaseinit.TIM_Prescaler = 			(uint16_t)((SystemCoreClock / 2) / WS2812_TIM_FREQ) - 1; //tim3 runs at half the speed of the system ->divide by 2
	TIM_TimeBaseInit(WS2812_TIM, &timbaseinit);

	TIM_OCStructInit(&timocinit);
	timocinit.TIM_OCMode = 					TIM_OCMode_PWM1;
	timocinit.TIM_OCPolarity = 				TIM_OCPolarity_High;
	timocinit.TIM_OutputState = 			TIM_OutputState_Enable;
	timocinit.TIM_Pulse = 					0;
	WS2812_TIM_OC_INIT(WS2812_TIM, &timocinit);

	WS2812_TIM_OC_PRECFG(WS2812_TIM, TIM_OCPreload_Enable);
	TIM_ARRPreloadConfig(WS2812_TIM, ENABLE);

	TIM_CCxCmd(WS2812_TIM, WS2812_TIM_CHANNEL, TIM_CCx_Enable);
	TIM_Cmd(WS2812_TIM, ENABLE);

	// DMA
	RCC_AHB1PeriphClockCmd(WS2812_RCC_DMA, ENABLE);
	TIM_DMACmd(WS2812_TIM, WS2812_DMA_SOURCE, ENABLE);
	DMA_ITConfig(WS2812_DMA_STREAM, DMA_IT_TC, ENABLE);

	// NVIC for DMA
	nvic_init_struc.NVIC_IRQChannel = DMA1_Stream7_IRQn;
	nvic_init_struc.NVIC_IRQChannelPreemptionPriority = 2;
	nvic_init_struc.NVIC_IRQChannelSubPriority = 2;
	nvic_init_struc.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&nvic_init_struc);

	WS2812startDMA();
}




// transfer framebuffer data to the timer
static void WS2812startDMA(void)
{
	if (ledBusy)		// last DMA is not finished
		return;

	ledBusy = 1;
	static DMA_InitTypeDef dma_init =
	{
			.DMA_BufferSize 		= WS2812_TIMERDMA_LEN,
			.DMA_Channel 			= WS2812_DMA_CHANNEL,
			.DMA_DIR 				= DMA_DIR_MemoryToPeripheral,
			.DMA_FIFOMode 			= DMA_FIFOMode_Disable,
			.DMA_FIFOThreshold 		= DMA_FIFOThreshold_HalfFull,
			.DMA_Memory0BaseAddr 	= (uint32_t) &ws2812timerValues[0],
			.DMA_MemoryBurst 		= DMA_MemoryBurst_Single,
			.DMA_MemoryDataSize 	= DMA_MemoryDataSize_HalfWord,
			.DMA_MemoryInc 			= DMA_MemoryInc_Enable,
			.DMA_Mode 				= DMA_Mode_Normal,
			.DMA_PeripheralBaseAddr = (uint32_t) &WS2812_TIM->WS2812_CCR,
			.DMA_PeripheralBurst 	= DMA_PeripheralBurst_Single,
			.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord,
			.DMA_PeripheralInc 		= DMA_PeripheralInc_Disable,
			.DMA_Priority 			= DMA_Priority_Medium
	};

	DMA_Cmd(WS2812_DMA_STREAM, DISABLE);
	while (DMA_GetCmdStatus(WS2812_DMA_STREAM) == ENABLE)
		; //wait
	DMA_Init(WS2812_DMA_STREAM, &dma_init);
	DMA_Cmd(WS2812_DMA_STREAM, ENABLE);
	TIM_DMACmd(WS2812_TIM, WS2812_DMA_SOURCE, ENABLE);
}




// gets called when dma transfer has completed
void DMA1_Stream7_IRQHandler(void)
{
	DMA_ClearITPendingBit(WS2812_DMA_STREAM, DMA_IT_TCIF7);
	DMA_Cmd(WS2812_DMA_STREAM, DISABLE);
	while (DMA_GetCmdStatus(WS2812_DMA_STREAM) == ENABLE)
		; //wait

	// need to disable this, otherwise some glitches can occur (first bit gets lost)
	TIM_DMACmd(WS2812_TIM, WS2812_DMA_SOURCE, DISABLE);

	ledBusy = 0;			// get ready for next transfer
}




void WS2812test(void)
{
	uint32_t i,j;
#define NR_TEST_PATTERNS	12
	uint8_t patterns[NR_TEST_PATTERNS][3] = {
			{0xf0,0x00,0x00},
			{0x00,0xf0,0x00},
			{0x00,0x00,0xf0},
			{0xf0,0xf0,0x00},
			{0xf0,0x00,0xf0},
			{0x00,0xf0,0xf0},
			{0xf0,0xf0,0xf0},
			{0x40,0x40,0x40},
			{0xf0,0xf0,0xf0},
			{0x40,0x40,0x40},
			{0xf0,0xf0,0xf0},
			{0x00,0x00,0x00},
	};

	for (i = 0; i < WS2812_NR_LEDS; i++) {
		ws2812ledRGB[i].R = 0;
		ws2812ledRGB[i].G = 0;
		ws2812ledRGB[i].B = 0;
	}

	for(j=0; j<NR_TEST_PATTERNS; j++)
	{
		for (i = 0; i < WS2812_NR_LEDS; i += 4)
		{
			int k = i;
			for (; k < i+4; k++)
			{
				ws2812ledRGB[k].R = patterns[j][0];
				ws2812ledRGB[k].G = patterns[j][1];
				ws2812ledRGB[k].B = patterns[j][2];
			}
			while (ledBusy)
				;
			WS2812update();
		}
		delay_ms(1000);
	}

//	init_ir();
//	uint32_t irkey = 0;
//	char c = '0';
//	j = 0;
//
//	while (1)
//	{
//		irkey = ir_getkeypress();
//		if(irkey) {
//			switch (irkey)
//			{
//				case 32204607:   //R
//				case 4218239797: //1
//					c = '7'; break;  //white balance red
//			}
//		}
//		if (c=='7')
//		{
//			j=j+1;
//			for (i = 0; i < WS2812_NR_LEDS; i++)
//			{
//				ws2812ledRGB[i].R = j;
//				ws2812ledRGB[i].G = j;
//				ws2812ledRGB[i].B = j;
//				while (ledBusy)
//					;
//				WS2812update();
//			}
//			c = '1';
//		}
//	}
}
